use tokio::sync::mpsc::UnboundedSender;
use color_eyre::Result;
use subxt::backend::legacy::rpc_methods::LegacyRpcMethods;

use crate::{action::Action, casper::CasperConfig};

pub async fn get_node_name(
    action_tx: &UnboundedSender<Action>,
    api: &LegacyRpcMethods<CasperConfig>,
) -> Result<()> {
    let maybe_node_name = api.system_name().await.ok();
    action_tx.send(Action::SetNodeName(maybe_node_name))?;
    Ok(())
}

pub async fn get_system_health(
    action_tx: &UnboundedSender<Action>,
    api: &LegacyRpcMethods<CasperConfig>,
) -> Result<()> {
    let (maybe_peers, is_syncing, should_have_peers) = api
        .system_health()
        .await
        .ok()
        .map_or((None, false, false), |health| (
                Some(health.peers),
                health.is_syncing,
                health.should_have_peers,
        ));
    action_tx.send(Action::SetSystemHealth(
            maybe_peers, 
            is_syncing, 
            should_have_peers))?;
    Ok(())
}

pub async fn get_genesis_hash(
    action_tx: &UnboundedSender<Action>,
    api: &LegacyRpcMethods<CasperConfig>,
) -> Result<()> {
    let maybe_genesis_hash = api
        .genesis_hash()
        .await
        .ok();
    action_tx.send(Action::SetGenesisHash(maybe_genesis_hash))?;
    Ok(())
}

pub async fn get_chain_name(
    action_tx: &UnboundedSender<Action>,
    api: &LegacyRpcMethods<CasperConfig>,
) -> Result<()> {
    let maybe_chain_name = api
        .system_chain()
        .await
        .ok();
    action_tx.send(Action::SetChainName(maybe_chain_name))?;
    Ok(())
}

pub async fn get_system_version(
    action_tx: &UnboundedSender<Action>,
    api: &LegacyRpcMethods<CasperConfig>,
) -> Result<()> {
    let maybe_system_version = api
        .system_version()
        .await
        .ok();
    action_tx.send(Action::SetChainVersion(maybe_system_version))?;
    Ok(())
}
